﻿/* Uncomment following line to turn on debugging of this operator. */
//#define DEBUG_EM

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * The exchange mutation operator. It randomly chooses two genes within
     * a chromosome with uniform probability, and then simply swaps them.
     * 
     * @author Tomas Tunys
     */
    public class CExchangeMutation : CBaseMutationOperator
    {
        /**
         * Creates new exchange mutation operator for the specified configuration.
         * 
         * @param configuration The configuration of a genetic algorithm that uses
         *                      this operator.
         * @param mutationRate  The mutation rate.
         */
        public CExchangeMutation(CConfiguration configuration, double mutationRate) : base(configuration, mutationRate)
        {
        }

        /**
         * Mutates the specified chromosome in a operator-specific way forming 
         * a new chromosome, which will then go to the next generation.
         * 
         * @param chromosome The chromosome to be mutated.
         */
        public override void apply(IChromosome chromosome)
        {
            if (shouldPerformMutation())
            {
                int i = 0, j = 0 ;

                /* Chooses two random positions within chromosome... */
                while (i == j)
                {
                    i = getConfiguration().getRandomGenerator().Next(chromosome.getSize()) ;
                    j = getConfiguration().getRandomGenerator().Next(chromosome.getSize()) ;
                }
#if DEBUG_EM
                Console.WriteLine("Exchange Mutation: " + (i + 1) + " <-> " + (j + 1)) ;
                printOutGenome(chromosome, (int)Math.Ceiling(Math.Log10(chromosome.getSize())), true) ;
#endif
                /* ... and swaps the genes in them. */
                swapGenes(chromosome, i, j) ;
#if DEBUG_EM
                printOutGenome(chromosome, (int)Math.Ceiling(Math.Log10(chromosome.getSize())), false) ;
#endif
            }
        }        
    }
}
