﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * An interface for implementation of a chromosome pool. The pool
     * may be used to store reusable chromosome instances, so instead
     * of creating new chromosome instances, the chromosomes from the
     * pool may be used which give a performance boost proportional
     * to the complexity of chromosome's construction.
     */
    public interface IChromosomePool
    {
        /**
         * Returns a chromosome from the pool. If the pool
         * is empty, null is returned.
         * 
         * @return A chromosome, or null, if the pool is
         *         empty.
         */
        IChromosome acquire() ;

        /**
         * Puts the specified chromosome into the pool so
         * it may be reused. 
         * 
         * @param chromosome The chromosome to be added
         *                   into the pool.
         */
        void release(IChromosome chromosome) ;

        /**
         * Sets the capacity of the pool. Any chromosome
         * added when the pool is at its maximum capacity
         * is discarded.
         * 
         * @param size The size of the pool.
         */
        void setMaximumSize(int size) ;

        /**
         * Empties the chromosome pool.
         */
        void clear() ;
    }
}
