﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MCScheduling.GeneticAlgorithm
{
    /**
     * Crossover operator is one of the tools of evolution. It is applied to
     * a couple of chromosomes, whose genes are exchanged accordingly, making
     * one or more offsprings which then go to the next generation.
     */
    public interface ICrossoverOperator
    {
        /**
         * Crossovers the specified couple of chromosomes in a operator-specific way
         * forming a new chromosomes, who should be added to the specified population.
         * 
         * @param daddy      The first chromosome entering into the crossover.
         * @param mommy      The second chromosome entering into the crossover.
         * @param population The population for the next generation. It may
         *                   contain some chromosomes that have been already chosen
         *                   for the next generation.
         */
        void apply(IChromosome daddy, IChromosome mommy, CPopulation population) ;
    }
}
