﻿using MCScheduling.SimulatedAnnealing ;

namespace MCScheduling.MixedCriticality.SA
{
    /**
     * The representation of a solution of an MC instance used by simulated
     * annealing algorithm.
     */
    public class CSimulatedAnnealingState : ICloneable<CSimulatedAnnealingState>
    {
        /** The array of indices to MC jobs of the instace, represented by this state. **/
        public int[] ScheduledJobIndex ;

        /** The start times of the jobs. **/
        public int[] StartTime ;

        /** The latenesses of the jobs. **/
        public int[] Lateness ;

        /**
         * Creates a new state for the simulated annealing algorithm.
         * 
         * @param scheduledJobIndices A "solution" of an MC instance in form of an array
         *                            of indices to its list of MC jobs.
         * @param startTimes          The precalculated list of start times of the jobs.
         * @param latenesses          The precalculated list of latenesses of the jobs.
         */
        public CSimulatedAnnealingState(int[] scheduledJobIndices, int[] startTimes, int[] latenesses)
        {
            ScheduledJobIndex = scheduledJobIndices ;
            StartTime = startTimes ;
            Lateness = latenesses ;
        }

        /**
         * Copies the data from the specified state into this one.
         * 
         * @param state The state to be cloned.
         */
        void ICloneable<CSimulatedAnnealingState>.Clone(CSimulatedAnnealingState state)
        {
            System.Array.Copy(state.ScheduledJobIndex, ScheduledJobIndex, ScheduledJobIndex.Length) ;
            System.Array.Copy(state.StartTime, StartTime, StartTime.Length) ;
            System.Array.Copy(state.Lateness, Lateness, Lateness.Length) ;
        }

        /**
         * Returns a shallow copy of this instance.
         * 
         * @return A shallow copy of this instance.
         */
        public object Clone()
        {
            return new CSimulatedAnnealingState((int[])ScheduledJobIndex.Clone(), (int[])StartTime, (int[])Lateness) ;
        }

        /**
         * Returns the number of MC jobs in the associated MC instance.
         */
        public int Length
        {
            get { return ScheduledJobIndex.Length ; }
        }
    }
}
