﻿using System ;
using System.Windows.Forms ;
using System.Runtime.InteropServices ;
using MCScheduling.MixedCriticality ;

namespace MCScheduling
{
    /**
     * Auxiliary class that binds its member methods to 
     * the same methods of kernel33.dll library.
     */
    public class Win32
    {
        /**
         * Creates a new console window and attaches this application
         * to it.
         * 
         * @return Returns true, if the console has been created, otherwise
         *         returns false.
         */
        [DllImport("kernel32.dll")]
        public static extern Boolean AllocConsole() ;

        /**
         * Destroys and detaches this application from the console
         * window previously created by calling AllocConsole().
         * 
         * @return Returns true, on success, otherwise, false.
         */
        [DllImport("kernel32.dll")]
        public static extern Boolean FreeConsole() ;

        /**
         * Attaches this application to already created console window
         * by a running process with PID dwProcessId.
         * 
         * @param dwProcessId The PID of the process to whose console
         *                    will the application tries to attach
         *                    itself.
         * 
         * @return Returns true, if the application has been attached,
         *         otherwise, returns false.
         */
        [DllImport("kernel32.dll")]
        public static extern Boolean AttachConsole(Int32 dwProcessId) ;
    }

    /**
     * The main application class.
     */
    static class Program
    {
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles() ;
            Application.SetCompatibleTextRenderingDefault(false) ;

            string[] arguments = Environment.GetCommandLineArgs() ;

            /* If there are no command line arguments, GUI is launched. */
            if (arguments.Length == 1)
            {
#if DEBUG
                Win32.AllocConsole() ;
#endif
                Application.Run(new MainWindowForm()) ;
            }
            /* otherwise, the output goes to console window. */
            else if(Win32.AttachConsole(-1))
            {
                /* Runs the solver with the specified arguments in the console. */
                (new CConsoleSolver(arguments)).run() ;
            }
            else
            {
                MessageBox.Show("Cannot attach the application to the console!", "Console", MessageBoxButtons.OK, MessageBoxIcon.Error) ;
            }
        }
    }
}
