﻿namespace MCScheduling.SimulatedAnnealing
{
    /**
     * The interface of a temperature cooler that is used to
     * decrease the temperature throughout the optimization
     * process of simulated annealing.
     */
    public interface ICooler
    {
        /**
         * Invoked when the simulated annealing optimization
         * is about to be executed.
         */
        void Reset() ;

        /**
         * Invoked in each iteration of the simulated annealing
         * algorithm.
         * 
         * @param temperature The current temperature of the simulated
         *                    annealing process.
         *
         * @return The temperature for the next iteration of the simulated
         *         annealing process.
         */
        double Cooldown(double temperature) ;
    }
}
